--[[
	HappyTotem!
	]]
	
-- key:value pairs for name:element. Totem name (as in spellbook) is the key,
-- element (corresponding to an element in the Elements[] element list) is the
-- value.
-- Used internally only. Earth/Fire/Water/Air do not need to be localised here.
TotemList = {	[HAPPYTOTEM_TOTEMNAME_STRENGTH_OF_EARTH]="Earth",	[HAPPYTOTEM_TOTEMNAME_STONESKIN]	="Earth",
				[HAPPYTOTEM_TOTEMNAME_EARTHBIND]		="Earth",	[HAPPYTOTEM_TOTEMNAME_STONECLAW]	="Earth",
				[HAPPYTOTEM_TOTEMNAME_TREMOR]			="Earth",	[HAPPYTOTEM_TOTEMNAME_EARTH_ELE]	="Earth",
				[HAPPYTOTEM_TOTEMNAME_SEARING]			="Fire",	[HAPPYTOTEM_TOTEMNAME_FIRE_NOVA]	="Fire",
				[HAPPYTOTEM_TOTEMNAME_MAGMA]			="Fire",	[HAPPYTOTEM_TOTEMNAME_FLAMETONGUE]	="Fire",
				[HAPPYTOTEM_TOTEMNAME_WRATH]			="Fire",	[HAPPYTOTEM_TOTEMNAME_FIRE_ELE]		="Fire",
				[HAPPYTOTEM_TOTEMNAME_FROST_RESIST]		="Fire",
				[HAPPYTOTEM_TOTEMNAME_HEALING_STREAM]	="Water",	[HAPPYTOTEM_TOTEMNAME_MANA_SPRING]	="Water",
				[HAPPYTOTEM_TOTEMNAME_MANA_TIDE]		="Water",	[HAPPYTOTEM_TOTEMNAME_POISON_CLEANSING]="Water",
				[HAPPYTOTEM_TOTEMNAME_DISEASE_CLEANSING]="Water",	[HAPPYTOTEM_TOTEMNAME_FIRE_RESIST]	="Water",
				[HAPPYTOTEM_TOTEMNAME_WINDFURY]			="Air",		[HAPPYTOTEM_TOTEMNAME_GROUNDING]	="Air",
				[HAPPYTOTEM_TOTEMNAME_WRATH_OF_AIR]		="Air",		[HAPPYTOTEM_TOTEMNAME_TRANQUIL_AIR]	="Air",
				[HAPPYTOTEM_TOTEMNAME_GRACE_OF_AIR]		="Air",		[HAPPYTOTEM_TOTEMNAME_WINDWALL]		="Air",
				[HAPPYTOTEM_TOTEMNAME_NATURE_RESIST]	="Air" };

-- key:value pairs of name:duration. Used for totem timer tracking.
-- I'd rather this was generated at launch, too, but the durations in the tooltips
-- aren't consistently formatted at all. Unless I can think of a better way these
-- are going to have to stay like this. There's no point writing a bunch of patterns
-- to cover all the edge cases if they might just get broken in a patch anyway.

TotemDuration = { [HAPPYTOTEM_TOTEMNAME_STRENGTH_OF_EARTH]=120, [HAPPYTOTEM_TOTEMNAME_STONESKIN]=120, [HAPPYTOTEM_TOTEMNAME_EARTHBIND]=45,
				[HAPPYTOTEM_TOTEMNAME_STONECLAW]=15, [HAPPYTOTEM_TOTEMNAME_TREMOR]=120, [HAPPYTOTEM_TOTEMNAME_EARTH_ELE]=120,
				[HAPPYTOTEM_TOTEMNAME_SEARING]=45, [HAPPYTOTEM_TOTEMNAME_FIRE_NOVA]=4, [HAPPYTOTEM_TOTEMNAME_MAGMA]=20,
				[HAPPYTOTEM_TOTEMNAME_FLAMETONGUE]=120, [HAPPYTOTEM_TOTEMNAME_WRATH]=120, [HAPPYTOTEM_TOTEMNAME_FIRE_ELE]=120,
				[HAPPYTOTEM_TOTEMNAME_FROST_RESIST]=120,
				[HAPPYTOTEM_TOTEMNAME_HEALING_STREAM]=120, [HAPPYTOTEM_TOTEMNAME_MANA_SPRING]=120, [HAPPYTOTEM_TOTEMNAME_MANA_TIDE]=12,
				[HAPPYTOTEM_TOTEMNAME_POISON_CLEANSING]=120, [HAPPYTOTEM_TOTEMNAME_DISEASE_CLEANSING]=120,
				[HAPPYTOTEM_TOTEMNAME_FIRE_RESIST]=120,
				[HAPPYTOTEM_TOTEMNAME_WINDFURY]=120, [HAPPYTOTEM_TOTEMNAME_GROUNDING]=45, [HAPPYTOTEM_TOTEMNAME_WRATH_OF_AIR]=120,
				[HAPPYTOTEM_TOTEMNAME_TRANQUIL_AIR]=120,[HAPPYTOTEM_TOTEMNAME_GRACE_OF_AIR]=120, [HAPPYTOTEM_TOTEMNAME_WINDWALL]=120,
				[HAPPYTOTEM_TOTEMNAME_NATURE_RESIST]=120 };

-- Totem texture list cache. Filled at login and updated when learning spells.
SpellTexture = {};
-- Totem spell ID cache. Filled at login and updated when learning spells.
SpellID = {};

-- Element list. These correspond to the four button IDs.
-- Used internally only. Does not need to be localised.
Elements = { "Earth", "Fire", "Water", "Air" };

-- Placed totem list. Placed totems stored in corresponing button IDs.
PlacedTotem = { };
PlacedTotemHealth = { };
PlacedTotemExpireTime = { 0, 0, 0, 0 };
PlacedTotemBuffName = { };

-- Main frame/mod loading function.
--  Just registers the main frame's events and sets up /tb handler.

function HappyTotem_OnLoad()
	if (not(UnitClass("player") == HAPPYTOTEM_SHAMAN_CLASS_NAME)) then
		this:Hide();
		return;
	end
	
	SLASH_HappyTotem1 = "/ht";
	SlashCmdList["HappyTotem"] = HappyTotem_SlashCommandHandler;
	
	this:RegisterEvent("PLAYER_LOGIN");
	this:RegisterEvent("LEARNED_SPELL_IN_TAB");
	this:RegisterEvent("CHARACTER_POINTS_CHANGED");
	this:RegisterEvent("UNIT_AURA");
end

-- Main frame event handler.
--  All this does right now is cause the totem spell ID/texture lists
--  to be generated when the player finishes logging in or learns a
--  new spell (since the spell IDs are based on spellbook location,
--  learning any new spell will jig them about).
--
--  In future I want to push the totem handling up into here because
--  firing four event handlers instead of one is making me angsty.

function HappyTotem_OnEvent(event, ...)
	if ( event == "PLAYER_LOGIN" ) then
		HappyTotem_FindTalentInfo();
		HappyTotem_FindTotemSpellIDs();
		
	elseif (event == "LEARNED_SPELL_IN_TAB") then
		HappyTotem_FindTotemSpellIDs();
	elseif (event == "CHARACTER_POINTS_CHANGED") then
		if (arg1 == -1) then
			HappyTotem_FindTalentInfo();
		end
	elseif (event == "UNIT_AURA") then
		if (arg1 ~= "player") then
			return;
		end
		
		HappyTotem_DoRangeCheck();
	end
end

function HappyTotem_DoRangeCheck()
	if (not HappyTotem_EnableRangeIndicator) then
		return;
	end
	
	-- Cycle through our buffs
	local buff, i, j;
	local name;
	local inRange = { false, false, false, false };
	local flash;
	
	i = 1;
	buff = UnitBuff("player", i);
	
--	DEFAULT_CHAT_FRAME:AddMessage("UNIT_AURA: Checking buffs...\n");
	while (buff) do
--		DEFAULT_CHAT_FRAME:AddMessage("Found buff: "..buff.."\n");
		for j = 1, 4 do
			if (buff == PlacedTotemBuffName[j]) then
				inRange[j] = true;
			end
		end
		i = i + 1;
		buff = UnitBuff("player", i);
	end
	
	for j = 1, 4 do
		flash = getglobal("TotemButton"..j.."Flash");
		if ((not inRange[j]) and PlacedTotemBuffName[j]) then
			flash:Show();
		else
			flash:Hide();
		end
	end
end

-- Overall frame update function.
--  Right now, this frame's job is to fade in the little tab jigger
--  at the top, or fade it out, depending on if the mouse is over
--  the frame.
function HappyTotem_OnUpdate(elapsed)
	local xPos, yPos = GetCursorPosition();
	local tab = getglobal("HappyTotemTab");

	if (MouseIsOver(this, 10, -10, -5, 5)) then
		if ( this.hover ) then
			if (this.oldX == xPos and this.oldy == yPos) then
				this.hoverTime = this.hoverTime + elapsed;
			else
				this.hoverTime = 0;
				this.oldX = xPos;
				this.oldy = yPos;
			end
			if (this.hoverTime > 0.5) then
				if ( not tab.hasBeenFaded ) then
					UIFrameFadeIn(tab, CHAT_FRAME_FADE_TIME);
					tab.oldAlpha = 1;
				end
				tab.hasBeenFaded = 1;
			end
		else
			-- Start hovering counter
			this.hover = 1;
			this.hoverTime = 0;
			tab.hasBeenFaded = nil;
			CURSOR_OLD_X, CURSOR_OLD_Y = GetCursorPosition();
		end
	elseif ( tab.hasBeenFaded and (not this.isMoving) ) then
		local fadeInfo = {};
		fadeInfo.mode = "OUT";
		fadeInfo.startAlpha = tab.oldAlpha;
		fadeInfo.timeToFade = CHAT_FRAME_FADE_TIME;
		fadeInfo.finishedArg1 = tab;
		fadeInfo.finishedArg2 = getglobal(this);
		fadeInfo.finishedFunc = HappyTotemTabFadeFinished;
		UIFrameFade(tab, fadeInfo);

		this.hover = nil;
		tab.hasBeenFaded = nil;
	end
end

function HappyTotemTabFadeFinished(tab, mainframe)
	tab:Hide();
end

-- Handle /tb commands.
--  at present, this only shows/hides the window. I don't really want to
--  have more than that at the moment. Maybe I'll add more eventually for
--  people who prefer typing arcane commands into the chat window over
--  clicking things (I sympathise).

function HappyTotem_SlashCommandHandler()
	if (HappyTotemFrame:IsVisible()) then
		HideUIPanel(HappyTotemFrame);
	else
		ShowUIPanel(HappyTotemFrame);
	end
end

--  Finds the spell IDs and textures of each totem, indexing them by name.
-- This function is called when the player loads up (not when the mod loads
-- because the spellbook won't be loaded then). It's also called when the
-- player learns a new spell or spell rank, because the IDs are based on
-- spell book location and a new spell messes that up.
function HappyTotem_FindTotemSpellIDs()
	local	i; -- iterator
	local	spellName, spellRank;
	
	i = 1;
	spellName, _ = GetSpellName(i, "BOOKTYPE_SPELL")
	
	while 1 do
		if (TotemList[spellName]) then
			SpellID[spellName] = i;
			SpellTexture[spellName] = GetSpellTexture(i, "BOOKTYPE_SPELL");
		end
		
		i = i + 1;
		spellName, _ = GetSpellName(i, "BOOKTYPE_SPELL");
		if (not spellName) then
			break;
		end
	end
end

ImpStoneclawPoints = 0;
ImpFireNovaPoints = 0;
--  Gets pertinent information on totem talents.
function HappyTotem_FindTalentInfo()
	_, _, _, _, ImpStoneclawPoints = GetTalentInfo(1, 3);
	_, _, _, _, ImpFireNovaPoints = GetTalentInfo(1, 9);
end

-- FIXME: is "Rank x" localised?
StoneclawTotemHealthByRank = {	["Rank 1"]=50, ["Rank 2"]=150, ["Rank 3"]=220, ["Rank 4"]=280, 
								["Rank 5"]=390,["Rank 6"]=480, ["Rank 7"]= 1315};

function HappyTotem_GetTotemHealthByNameAndRank(name, rank)
	if (name == HAPPYTOTEM_TOTEMNAME_STONECLAW) then
		local health;
		health = StoneclawTotemHealthByRank[rank];
		health = health + ImpStoneclawPoints * 0.25;
		return health;
	elseif (name == HAPPYTOTEM_TOTEMNAME_EARTH_ELE) then
		return 7400;
	elseif (name == HAPPYTOTEM_TOTEMNAME_FIRE_ELE) then
		return 3888;
	else
		return 5;
	end
end

-- FIXME: is "Rank x" localised?
SearingTotemDurationByRank = {	["Rank 1"]=30, ["Rank 2"]=35, ["Rank 3"]=40, ["Rank 4"]=45,
								["Rank 5"]=50, ["Rank 6"]=55, ["Rank 7"]=60 };
function HappyTotem_GetTotemDurationByNameAndRank(name, rank)
	if (name == HAPPYTOTEM_TOTEMNAME_SEARING) then
		return SearingTotemDurationByRank[rank];
	elseif (name == HAPPYTOTEM_TOTEMNAME_FIRE_NOVA) then
		return 4 - ImpFireNovaPoints;
	else
		return TotemDuration[name];
	end
end

--[[		Totem Button Stuff		]]

-- Button loading function.
--  Pretty self-explanatory.
function TotemButton_OnLoad()
	if (UnitClass("player") ~= HAPPYTOTEM_SHAMAN_CLASS_NAME) then
		this:Hide();
		return;
	end
	this.flashing = 0;
	this.flashtime = 0;
	TotemButton_Update();
	this:SetAttribute("type", "action");
	this:SetAttribute("checkselfcast", true);
	this:SetAttribute("useparent-unit", true);
	this:RegisterForClicks("AnyUp");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("UPDATE_BINDINGS");
	this:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	this:RegisterEvent("CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS");
	this:RegisterEvent("CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE");
	this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS");
	this:RegisterEvent("CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE");
	this:RegisterEvent("PLAYER_DEAD");
	
	
	local cooldownFrame = getglobal(this:GetName() .. "Cooldown");
	cooldownFrame:SetReverse();
end

function TotemButton_OnEvent(event, ...)
	if ( event == "PLAYER_ENTERING_WORLD") then
		TotemButton_Update();
	elseif ( event == "PLAYER_DEAD" ) then
		TotemButton_ResetButton();
	elseif ( event == "UNIT_SPELLCAST_SUCCEEDED" ) then
		if (arg1 == "player") then
			TotemButton_UpdateTotemDisplayBySpellName(arg2, arg3);
		end
	elseif (event == "CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS" or
			event == "CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE" or
			event == "CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE" or
			event == "CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS") then
		local totem, damage = string.match(arg1, HAPPYTOTEM_PATTERN_TOTEM_HIT);
		if (totem) then
			local myID = this:GetID();
			if (PlacedTotem[myID] == totem) then
				-- DEFAULT_CHAT_FRAME:AddMessage("Enemy damaged my "..totem.." for "..damage.." damage!\n");
				PlacedTotemHealth[myID] = PlacedTotemHealth[myID] - damage;
				if (PlacedTotemHealth[myID] <= 0) then
					TotemButton_ResetButton();
				end
			end
		end
	elseif ( event == "ACTIONBAR_UPDATE_STATE" ) then
		TotemButton_UpdateState();
	elseif ( event == "ACTIONBAR_UPDATE_USABLE" or event == "UPDATE_INVENTORY_ALERTS" or event == "ACTIONBAR_UPDATE_COOLDOWN" ) then
		TotemButton_UpdateUsable();
		TotemButton_UpdateCooldown();
	elseif ( event == "CRAFT_SHOW" or event == "CRAFT_CLOSE" or event == "TRADE_SKILL_SHOW" or event == "TRADE_SKILL_CLOSE" ) then
		TotemButton_UpdateState();
	end
end

function TotemButton_ResetButton()
	local icon = getglobal(this:GetName().."Icon");
	local cooldownFrame = getglobal(this:GetName() .. "Cooldown");
	local flashFrame = getglobal(this:GetName() .. "Flash");
	local id = this:GetID();
	icon:Hide();
	cooldownFrame:Hide();
	flashFrame:Hide();
	-- FIXME: does this texture work in international clients?
	this:SetNormalTexture("Interface\\Buttons\\UI-Quickslot");
	PlacedTotem[id] = "none";
	PlacedTotemExpireTime[id] = 0;
	PlacedTotemBuffName[id] = nil;
end

function TotemButton_UpdateTotemDisplayBySpellName(name, rank)
	if (name == HAPPYTOTEM_SPELLNAME_TOTEMIC_CALL) then
		TotemButton_ResetButton();
	end
	
	if (not TotemList[name]) then
		return;
	end
	
	local myID = this:GetID();
	
	if (TotemList[name] == Elements[myID])	then
		local icon = getglobal(this:GetName().."Icon");
		local cooldownFrame = getglobal(this:GetName().."Cooldown");
		local flashFrame = getglobal(this:GetName().."Flash");
		
		icon:SetTexture(SpellTexture[name]);
		icon:Show();
		PlacedTotem[myID] = name;
		PlacedTotemHealth[myID] = HappyTotem_GetTotemHealthByNameAndRank(name, rank);
		PlacedTotemBuffName[myID] = TotemBuffName[name];
		local duration = HappyTotem_GetTotemDurationByNameAndRank(name, rank);
		CooldownFrame_SetTimer(cooldownFrame, GetTime(), duration, 1);
		flashFrame:Hide(); -- totem should be assumed to be in range of you when dropped
		PlacedTotemExpireTime[myID] = GetTime() + duration + 0.5;
		-- FIXME: does this work right in localised clients?
		this:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2");
	end
end

function TotemButton_SetTooltip()

end

function TotemButton_UpdateState(button)

end

function TotemButton_UpdateUsable(button)

end

function TotemButton_UpdateCooldown(button)
	
end

function TotemButton_UpdateFlash(button)

end


function TotemButton_Update(arg1)
	TotemButton_ResetButton();

	this:RegisterEvent("ACTIONBAR_UPDATE_STATE");
	this:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
	this:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
	this:RegisterEvent("UPDATE_INVENTORY_ALERTS");
	this:RegisterEvent("PLAYER_AURAS_CHANGED");
	this:RegisterEvent("PLAYER_TARGET_CHANGED");
	this:RegisterEvent("UNIT_INVENTORY_CHANGED");
	this:RegisterEvent("CRAFT_SHOW");
	this:RegisterEvent("CRAFT_CLOSE");
	this:RegisterEvent("TRADE_SKILL_SHOW");
	this:RegisterEvent("TRADE_SKILL_CLOSE");
	this:RegisterEvent("PLAYER_ENTER_COMBAT");
	this:RegisterEvent("PLAYER_LEAVE_COMBAT");
	this:RegisterEvent("START_AUTOREPEAT_SPELL");
	this:RegisterEvent("STOP_AUTOREPEAT_SPELL");

	if ( not this:GetAttribute("statehidden") ) then
		this:Show();
	end
	TotemButton_UpdateState();
	TotemButton_UpdateUsable();
	TotemButton_UpdateCooldown();
	TotemButton_UpdateFlash();

	if ( GameTooltip:IsOwned(this) ) then
		TotemButton_SetTooltip();
	else
		this.updateTooltip = nil;
	end
end

function TotemButton_OnUpdate(elapsed)
	local myID = this:GetID();
	if ( PlacedTotemExpireTime[myID] > 0 ) then
		if ( PlacedTotemExpireTime[myID] < GetTime()) then
			TotemButton_ResetButton();
		end
	end

	if ( not this.updateTooltip ) then
		return;
	end

	this.updateTooltip = this.updateTooltip - elapsed;
	if ( this.updateTooltip > 0 ) then
		return;
	end

	if ( GameTooltip:IsOwned(this) ) then
		TotemButton_SetTooltip();
	else
		this.updateTooltip = nil;
	end
end

--					TAB FUNCTIONS					--

function HappyTotemTab_OnClick(button)
	local frame = getglobal("HappyTotemFrame");

	frame:StopMovingOrSizing();
	frame.isMoving = false;
	ToggleDropDownMenu(1, nil, getglobal(this:GetName().."DropDown"), this:GetName(), 0, 0);
	return;

end

function HappyTotemOptionsDropDown_OnLoad()
	UIDropDownMenu_Initialize(this, HappyTotemOptionsDropDown_Initialize, "MENU");
	UIDropDownMenu_SetButtonWidth(50);
	UIDropDownMenu_SetWidth(50);
end

HappyTotem_EnableRangeIndicator = true;
function HappyTotemOptionsDropDown_Initialize()
	info            = {};
	info.text       = HAPPYTOTEM_MENU_RANGE_CHECK_STRING;
	info.checked    = HappyTotem_EnableRangeIndicator;
	info.func       = HappyTotemOptionsDropDown_ToggleRangeIndicator;

	-- Add the above information to the options menu as a button.
	UIDropDownMenu_AddButton(info);
end

function HappyTotemOptionsDropDown_ToggleRangeIndicator()
	if (HappyTotem_EnableRangeIndicator) then
		-- disable indicator
		local flash;
		for i = 1, 4 do
			flash = getglobal("TotemButton"..i.."Flash");
			flash:Hide();
		end
		HappyTotem_EnableRangeIndicator = false;
	else
		-- enable indicator
		HappyTotem_EnableRangeIndicator = true;
		HappyTotem_DoRangeCheck();
	end
end
